local SpellAlerter = LibStub("AceAddon-3.0"):NewAddon("SpellAlerter","AceEvent-3.0","AceConsole-3.0")
local	L, SM = LibStub("AceLocale-3.0"):GetLocale("SpellAlerter", true), LibStub("LibSharedMedia-3.0")

do
	local function AddSound(soundName,soundFile) SM:Register("Sound",soundName,soundFile) end
	AddSound("Bell Toll Alliance", "Sound\\Doodad\\BellTollAlliance.wav") 
	AddSound("Bell Toll Horde", "Sound\\Doodad\\BellTollHorde.wav")
	AddSound("Rubber Ducky", "Sound\\Doodad\\Goblin_Lottery_Open01.wav")
	AddSound("Cartoon FX", "Sound\\Doodad\\Goblin_Lottery_Open03.wav")
	AddSound("Explosion", "Sound\\Doodad\\Hellfire_Raid_FX_Explosion05.wav")
	AddSound("Shing!", "Sound\\Doodad\\PortcullisActive_Closed.wav")
	AddSound("Wham!", "Sound\\Doodad\\PVP_Lordaeron_Door_Open.wav")
	AddSound("Simon Chime", "Sound\\Doodad\\SimonGame_LargeBlueTree.wav")
	AddSound("War Drums", "Sound\\Event Sounds\\Event_wardrum_ogre.wav")
	AddSound("Cheer", "Sound\\Event Sounds\\OgreEventCheerUnique.wav")
	AddSound("Humm", "Sound\\Spells\\SimonGame_Visual_GameStart.wav")
	AddSound("Short Circuit", "Sound\\Spells\\SimonGame_Visual_BadPress.wav")
	AddSound("Fel Portal", "Sound\\Spells\\Sunwell_Fel_PortalStand.wav")
	AddSound("Fel Nova", "Sound\\Spells\\SeepingGaseous_Fel_Nova.wav")
	AddSound("You Will Die!", "Sound\\Creature\\CThun\\CThunYouWillDIe.wav")
end

function SA_AddSound(soundName,soundFile)
	assert(type(soundName) == "string","Bad argument #1. Expected a string")
	assert(type(soundFile) == "string","Bad argument #2. Expected a string")
	SM:Register("sound",soundName,"Interface\\Addons\\SpellAlerter\\sounds\\"..soundFile) 
end

local profile, alert
local SpellCasts, EnemyBuffs, FriendlyDebuffs, Ignores
local PlayerName 
local PlayerGUID

local SPELLCASTS = "spellcasts"
local ENEMYBUFFS = "enemybuffs"
local FRIENDLYDEBUFFS = "friendlydebuffs"

local ListSelect, ListSelect2

local GetSpellInfo = GetSpellInfo
local UnitExists = UnitExists
local UnitName = UnitName
local UnitGUID = UnitGUID
local PlaySoundFile = PlaySoundFile
local pairs = pairs
local ipairs = ipairs
local unpack = unpack
local select = select
local match = string.match
local wipe = table.wipe

local ClassColors = {}
local function UpdateClassHexes()
	for class,color in pairs(CUSTOM_CLASS_COLORS or RAID_CLASS_COLORS) do
		ClassColors[class] = ("|cff%02x%02x%02x"):format(color.r * 255, color.g * 255, color.b * 255)
	end
end
UpdateClassHexes()
if CUSTOM_CLASS_COLORS then CUSTOM_CLASS_COLORS:RegisterCallback(UpdateClassHexes) end


local Defaults = {
	global = {
		Enabled = true,
		Lock = true,
		Minimap = {},
	},
	profile = {
		Positions = { SpellAlerterFrameAnchor =  {"CENTER",nil,"CENTER",0,190} },
		CategorySelect = "SpellCasts",
		TargetOnly = false,
		ShowIcon = true,
		SpellNames = false,
		ShowTarget = true,
		ClassColored = true,
		TargetGraphic = "arrow",
		HoldTime = 1,
		IconSize = 40,
		Font = SM:GetDefault("font"),
		FontSize = 40,
		YouText = "YOU",
		SpellCastsPlayersOnly = true,
		
	},
}

local CategoryValues = {
	SpellCasts = L["Spell Casts"],
	EnemyBuffs = L["Enemy Buffs"],
	FriendlyDebuffs = L["Friendly Debuffs"]
}

local colors = {
	YELLOW = {1,1,0},
	PINK = {1,0,1},
	GREEN = {0,1,0},
	RED = {1,0,0},
	WHITE = {1,1,1},
	VIOLET = {0.55,0,1},
	TAN = {0.82,0.71,0.55},
	TEAL = {0,0.5,0.5},
	TURQUOISE = {0.19,0.84,0.78},
	PEACH = {1,0.9,0.71},
	INDIGO = {0,0.25,0.71},
	GREY = {0.5,0.5,0.5},
	AQUA = {0,1,1},
	ORANGE = {1,0.65,0},
	BLUE = {0,0,1},
}

local colorHexes = {
	YELLOW =	   "|cffffff00"..L["Yellow"].."|r",
	PINK = 		"|cffff00ff"..L["Pink"].."|r",
	GREEN = 		"|cff00ff00"..L["Green"].."|r",
	RED = 		"|cffff0000"..L["Red"].."|r",
	WHITE = 		"|cffffffff"..L["White"].."|r",
	VIOLET = 	"|cff8b00ff"..L["Violet"].."|r",
	TAN = 		"|cffd2b48c"..L["Tan"].."|r",
	TEAL = 		"|cff008080"..L["Teal"].."|r",
	TURQUOISE = "|cff30d5c8"..L["Turquoise"].."|r",
	PEACH = 	   "|cffffe5b4"..L["Peach"].."|r",
	INDIGO =    "|cff00416a"..L["Indigo"].."|r",
	GREY =      "|cff808080"..L["Grey"].."|r",
	AQUA =      "|cff00ffff"..L["Aqua"].."|r",
	ORANGE =    "|cffffa500"..L["Orange"].."|r",
	BLUE =      "|cff0000ff"..L["Blue"].."|r",
}

function SpellAlerter:OpenConfig()
	LibStub("AceConfigDialog-3.0"):Open("SpellAlerter")
end

function SpellAlerter:ProfileChanged()
	self:AddDefaultSpells()
	ListSelect,ListSelect2 = nil,nil
	profile = self.db.profile
	SpellCasts = self.db.profile.SpellCasts
	FriendlyDebuffs = self.db.profile.FriendlyDebuffs
	EnemyBuffs = self.db.profile.EnemyBuffs
	Ignores = self.db.profile.Ignores
end

local function AddSpellInfo(list,keysame)
	local temp = {}
	for spellid,data in pairs(list) do 
		local spellName = GetSpellInfo(spellid)
		if spellName then temp[spellName] = keysame and spellName or data end
	end
	wipe(list)
	for spellName,data in pairs(temp) do list[spellName] = data end
end


function SpellAlerter:AddDefaultSpells()
	if not self.db.profile.Ignores then
		self.db.profile.Ignores =  {
		-- Heals
			-------------------------------------
				[26983] = true, -- Tranquility

		-- CC
			-------------------------------------
				-- Hunter
				[14311] = true, -- Freezing Trap
				[13809] = true, -- Frost Trap
				[34600] = true, -- Snake Trap
				-- Warlock
				[17928] = true, -- Howl of Terror
				-- Warrior
				[23920] = true, -- Spell Reflection
				
		-- Destroyable
			-------------------------------------
				[16190] = true, -- Mana Tide Totem
				[8177] = true,  -- Grounding Totem
				[61657] = true, -- Fire Nova Totem
				
		-- Damage
			-------------------------------------

		-- Ressurects/Summons
			-------------------------------------
			-- Hunter
				[982] = true,   -- Revive Pet
			-- Warlock
				[712] = true,   -- Summon Succubus
				[30146] = true, -- Summon Felguard
				[691] = true,   -- Summon Felhunter
				[697] = true,   -- Summon Voidwalker
		}
		AddSpellInfo(self.db.profile.Ignores,true)
	end
	
	if not self.db.profile.SpellCasts then
		self.db.profile.SpellCasts =  {
			-- Heals
			-------------------------------------
				-- Priests
				[25235] = {Color = "YELLOW"}, -- Flash Heal
				[25213] = {Color = "YELLOW"}, -- Greater Heal
				[32546] = {Color = "YELLOW"}, -- Binding Heal
				-- Druid
				[26979] = {Color = "YELLOW"}, -- Healing Touch
				[26980] = {Color = "YELLOW"}, -- Regrowth
				[26983] = {Color = "YELLOW"}, -- Tranquility
				-- Shaman 
				[25396] = {Color = "YELLOW"}, -- Healing Wave
				[25420] = {Color = "YELLOW"}, -- Lesser Healing Wave
				-- Paladin
				[27137] = {Color = "YELLOW"}, -- Flash of Light
				[27136] = {Color = "YELLOW"}, -- Holy Light
			-- CC
			-------------------------------------
				-- Priest
				[605] =   {Color = "PINK"},   -- Mind Control
				-- Druid
				[26989] = {Color = "PINK"},   -- Entangling Roots
				[33786] = {Color = "PINK"},   -- Cyclone
				[18658] = {Color = "PINK"}, 	-- Hibernate
				-- Hunter
				[14311] = {Color = "PINK"},   -- Freezing Trap
				[13809] = {Color = "PINK"},   -- Frost Trap
				[34600] = {Color = "PINK"},   -- Snake Trap
				-- Mage
				[12826] = {Color = "PINK"},   -- Polymorph
				-- Warlock
				[6215] =  {Color = "PINK"},   -- Fear
				[17928] = {Color = "PINK"},   -- Howl of Terror
				-- Warrior
				[23920] = {Color = "PINK"},   -- Spell Reflection
				-- Shaman
				[51514] = {Color = "PINK"},   -- Hex
				
			-- Destroyable
			-------------------------------------
				[16190] = {Color = "GREEN"},  -- Mana Tide Totem
				[8177] =  {Color = "GREEN"},  -- Grounding Totem
				[61657] = {Color = "GREEN"},  -- Fire Nova Totem
				
			-- Damage
			-------------------------------------
				-- Priest
				[8129]  = {Color = "RED"}, 	-- Mana Burn
				[48123] = {Color = "RED"},    -- Smite
				[48135] = {Color = "RED"},    -- Holy Fire
				[48127] = {Color = "RED"},    -- Mind Blast
				[48156] = {Color = "RED"},    -- Mind Flay
				[48160] = {Color = "RED"},    -- Vampiric Touch
				-- Mage
				[38697] = {Color = "RED"},    -- Frostbolt
				[38692] = {Color = "RED"},    -- Fireball
				[33938] = {Color = "RED"},    -- Pyroblast
				-- Shaman
				[25449] = {Color = "RED"},    -- Lightning Bolt
				[25442] = {Color = "RED"},    -- Chain Lightning
				[60043] = {Color = "RED"},    -- Lava Burst
				-- Warlock
				[47810] = {Color = "RED"},    -- Immolate
				[27209] = {Color = "RED"},    -- Shadow Bolt
				[30545] = {Color = "RED"},    -- Soul Fire
				[59164] = {Color = "RED"},    -- Haunt
				[47843] = {Color = "RED"},    -- Unstable Affliction
				[59172] = {Color = "RED"},    -- Chaos Bolt
		-- Ressurects/Summons
		-------------------------------------
			-- Druid
				[50763] = {Color = "GREY"},   -- Revive
			-- Paladin
				[48950] = {Color = "GREY"},   -- Redemption
			-- Priest
				[48171] = {Color = "GREY"},   -- Resurrect
			-- Hunter
				[982] =   {Color = "GREY"},   -- Revive Pet
			-- Warlock
				[712] =   {Color = "GREY"},   -- Summon Succubus
				[30146] = {Color = "GREY"},   -- Summon Felguard
				[691] =   {Color = "GREY"},   -- Summon Felhunter
				[697] =   {Color = "GREY"},   -- Summon Voidwalker
			-- Shaman
				[49277] = {Color = "GREY"},   -- Ancestral Spirit
		}
		AddSpellInfo(self.db.profile.SpellCasts)
	end
	if not self.db.profile.EnemyBuffs then
		self.db.profile.EnemyBuffs = {
			-- Clearable
			-------------------------------------
				-- Druid
				[29166] = {Color = "PEACH"},  -- Innervate
				[17116] = {Color = "PEACH"},  -- Nature's Swiftness
				-- Shaman
				[32594] = {Color = "PEACH"},  -- Earth Shield
				-- Paladin
				[1044] =  {Color = "VIOLET"}, -- Hand of Freedom
				[10278] = {Color = "VIOLET"}, -- Hand of Protection
				[6940] =  {Color = "VIOLET"}, -- Hand of Sacrifice
				-- Priest
				[10060] = {Color = "PEACH"},  -- Power Infusion
				-- Mage 
				[12051] = {Color = "PEACH"},  -- Evocation
				-- Warlock
				[18708] = {Color = "PEACH"},  -- Fel Domination
				-- Paladin
				[53654] = {Color = "PEACH"},  -- Beacon of Light
		}
		AddSpellInfo(self.db.profile.EnemyBuffs)
	end
	if not self.db.profile.FriendlyDebuffs then
		self.db.profile.FriendlyDebuffs = {
			-- Clearable
			-------------------------------------
				[3034] = {Color = "AQUA"},    -- Viper Sting
		}
		AddSpellInfo(self.db.profile.FriendlyDebuffs)
	end
end

function SpellAlerter:OnInitialize()
	self.frame = CreateFrame("Frame","SpellAlerterFrame",UIParent)
	self.db = LibStub("AceDB-3.0"):New("SpellAlerterDB",Defaults,"Default")
	self:ProfileChanged()

	self:RegisterChatCommand("sa",self.OpenConfig)
	self:RegisterChatCommand("spellalerter",self.OpenConfig)

	self.db.RegisterCallback(self,"OnProfileChanged","ProfileChanged")
	self.db.RegisterCallback(self,"OnProfileCopied","ProfileChanged")
	self.db.RegisterCallback(self,"OnProfileReset","ProfileChanged")
	self.db.RegisterCallback(self,"OnProfileDeleted","ProfileChanged")

	self:InitializeOptions()
	self.options.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	self.options.args.profile.order = 500
	LibStub("AceConfig-3.0"):RegisterOptionsTable("SpellAlerter", self.options)

	self.Launcher = LibStub("LibDataBroker-1.1"):NewDataObject("SpellAlerter", 
	{
		type = "launcher",
		icon = "Interface\\Icons\\Spell_Fire_Flare",
		OnClick = function(_, button)
			SpellAlerter:OpenConfig()
		end,
		OnTooltipShow = function(tooltip)
			tooltip:AddLine("Spell Alerter")
			tooltip:AddLine(L["|cff99ff33Click|r to open the config"])
		end,
	})
	LibStub("LibDBIcon-1.0"):Register("SpellAlerter",self.Launcher,self.db.global.Minimap)

	ChatFrame1:AddMessage("Spell Alerter by Kollektiv. Type /sa or /spellalerter to open options",0,1,0)
	self:SetEnabledState(self.db.global.Enabled)
end

function SpellAlerter:OnEnable()
	self.frame:Show()
	alert = alert or self:CreateAlert()
	PlayerName = UnitName("player")
	self:ApplySettings()
	self:ApplyLock()
	self:LoadPositions()
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	PlayerGUID = PlayerGUID or UnitGUID("player")
end

function SpellAlerter:OnDisable()
	LibStub("LibDBIcon-1.0"):Hide("SpellAlerter")
	self.frame:Hide()
	self:UnregisterAllEvents()
end

function SpellAlerter:SavePosition(anchor)
	profile.Positions[anchor:GetName()] = {anchor:GetPoint()}
end

function SpellAlerter:LoadPositions()
	for name,pos in pairs(profile.Positions) do
		local f = _G[name]
		if f then f:ClearAllPoints() f:SetPoint(unpack(pos)) end
	end
end

function SpellAlerter:CreateAlert()
	alert = CreateFrame("Frame","SpellAlerterFrameAlert",self.frame)
	alert.anchor = self:GetAnchor()
	alert:SetWidth(1); alert:SetHeight(1)
	alert:SetPoint("CENTER",alert.anchor,"CENTER")
	alert:SetFrameLevel(alert.anchor:GetFrameLevel()+1)
	alert.text = alert:CreateFontString(nil,"ARTWORK")
	alert.text2 = alert:CreateFontString(nil,"ARTWORK")
	alert.icon = alert:CreateTexture(nil,"ARTWORK")
	alert.icon:SetTexCoord(0.07,0.93,0.07,0.93)
	alert.icon:SetPoint("RIGHT",alert.text,"LEFT")
	alert.arrow = alert:CreateTexture(nil,"ARTWORK")
	alert.arrow:SetTexture("Interface\\Addons\\SpellAlerter\\Arrow")
	alert.arrow:SetHeight(42)
	alert.arrow:SetWidth(48)
	return alert
end

function SpellAlerter:GetAnchor()
	local anchor = CreateFrame("Frame",self.frame:GetName().."Anchor",self.frame)
	anchor:SetWidth(100)
	anchor:SetHeight(15)
	anchor.bg = anchor:CreateTexture(nil,"BACKGROUND")
	anchor.bg:SetTexture(SM:Fetch("background","Solid"))
	anchor.bg:SetAllPoints(true)
	anchor.bg:SetVertexColor(0,0,0,0.33)
	anchor.text = anchor:CreateFontString(nil,"ARTWORK")
	anchor.text:SetFont(SM:Fetch("font"),8)
	anchor.text:SetText("Spell Alerter Anchor")
	anchor.text:SetPoint("CENTER")
	anchor:SetMovable(true)
	anchor:EnableMouse(true)
	anchor:SetScript("OnMouseDown",function(self) self:StartMoving() end)
	anchor:SetScript("OnMouseUp",function(self) self:StopMovingOrSizing() SpellAlerter:SavePosition(self) end)
	return anchor
end

local uids = {}
local _uids = {}
local Levels = 3

do
	local function AddIds(uid,lower,upper)
		if lower and upper then
			for i=lower,upper do
				uids[#uids+1] = uid..i
			end
		else
			uids[#uids+1] = uid
		end
	end
	
	AddIds("target")
	AddIds("arena",1,5)
	AddIds("focus")
	AddIds("party",1,4)
	AddIds("pettarget")
	AddIds("partypet",1,4)
	AddIds("raid",1,40)
end

local function resetuids()
	wipe(_uids)
	for k,v in ipairs(uids) do _uids[k] = v end
end

resetuids()

-- returns id,nextid
local function FindTargetInfo(name)
	for i=1,Levels do
		for k,id in ipairs(_uids) do
			local nextid = id.."target"
			if id ~= "" and UnitExists(id) then 
				if UnitName(id) == name then
					resetuids()
					return id,nextid 
				else
					_uids[k] = nextid
				end
			else
				_uids[k] = ""
			end
		end
	end
	resetuids()
end

local function OnUpdate(self,elapsed)
	self.elapsed = self.elapsed + elapsed
	if self.elapsed <= 0.1 then
		self:SetAlpha(1-((0.1-self.elapsed)/0.1))
	elseif self.elapsed <= 0.1 + profile.HoldTime then
		self:SetAlpha(1)
	elseif self.elapsed <= 0.2 + profile.HoldTime then
		self:SetAlpha((profile.HoldTime + 0.2 - self.elapsed)/0.1)
	else
		self:SetAlpha(0)
		self:SetScript("OnUpdate",nil)
	end
end

local function StartOnUpdate()
	alert.elapsed = 0
	alert:SetScript("OnUpdate",OnUpdate)
end

function SpellAlerter:ApplyLock()
	if self.db.global.Lock then
		alert.anchor:Hide()
	else
		alert.anchor:Show()
	end
end

function SpellAlerter:ApplyMinimap()
	LibStub("LibDBIcon-1.0")[self.db.global.Minimap.hide and "Hide" or "Show"](LDBIcon, "SpellAlerter")
end

local function FixPoints(self,dst,same)
	alert.text:ClearAllPoints()
	alert.text2:ClearAllPoints()
	alert.arrow:ClearAllPoints()
	if same then
		alert.text:SetPoint("CENTER",alert.anchor,"CENTER")
		alert.arrow:SetPoint("LEFT",alert.text,"RIGHT")
		alert.text2:Hide()
		alert.arrow:SetTexCoord(0.01,(512/9.1)/512,((512/12)*2.9)/512,((512/12)*3.9)/512)
		alert.arrow:Show()
	elseif dst ~= "NONE" then
		alert.arrow:SetPoint("CENTER",alert.anchor,"CENTER")
		alert.text:SetPoint("RIGHT",alert.arrow,"LEFT")
		alert.text2:SetPoint("LEFT",alert.arrow,"RIGHT")
		alert.arrow:SetTexCoord(0,0.109375,0.73828125,0.8203125)
		alert.arrow:Show()
		alert.text2:Show()
	else
		alert.text:SetPoint("CENTER",alert.anchor,"CENTER")
		alert.arrow:Hide()
		alert.text2:Hide()
	end
end


function SpellAlerter:ApplySettings()
	alert.text:SetFont(SM:Fetch("font",profile.Font),profile.FontSize,"THICKOUTLINE")
	alert.text2:SetFont(SM:Fetch("font",profile.Font),profile.FontSize,"THICKOUTLINE")
	alert.icon:SetWidth(profile.IconSize)
	alert.icon:SetHeight(profile.IconSize)

	alert.text:ClearAllPoints()
	alert.text:SetPoint("CENTER",alert.anchor,"CENTER")
	alert.text2:Hide()
	alert.arrow:Hide()
	if profile.ShowIcon then alert.icon:Show() else alert.icon:Hide() end
	if profile.ShowTarget and profile.TargetGraphic == "arrow" then
		alert.FixPoints = FixPoints
	else
		alert.FixPoints = nil
	end
end

local function WrapClassColor(s,id)
	if not id or not UnitExists(id) then return s end
	local color = ClassColors[select(2,UnitClass(id))]
	if color then s = color..s.."|r" end
	return s
end

function SpellAlerter:FormatInfo(src,spellName,icon,sound,cat,r,g,b,forcedDst)
	if sound then PlaySoundFile(SM:Fetch("sound",sound)) end
	local srcID,dstID,dst
	srcID,dstID = FindTargetInfo(src)
	dst = forcedDst or "NONE"
	if cat == SPELLCASTS and not Ignores[spellName] and UnitExists(dstID) then 
		dst = PlayerGUID == UnitGUID(dstID) and profile.YouText or UnitName(dstID) 
	end
	local same = dst == src
	if profile.ClassColored then
		src = WrapClassColor(src,srcID)
		dst = dst ~= "NONE" and WrapClassColor(dst,dstID) or dst
	end
	src = profile.SpellNames and src.." - "..spellName or src

	if profile.ShowTarget and profile.TargetGraphic == "text" then
		if same then alert.text:SetText(src.." <")
		else alert.text:SetText(dst ~= "NONE" and src.." > "..dst or src) end
	else alert.text:SetText(src) end
	alert.text2:SetText(dst)
	alert.icon:SetTexture(icon)
	alert.text:SetVertexColor(r,g,b)
	alert.text2:SetVertexColor(r,g,b)
	alert.arrow:SetVertexColor(r,g,b)
	if alert.FixPoints then alert:FixPoints(dst,same) end
	StartOnUpdate()
end


local COMBATLOG_TARGET	= COMBATLOG_OBJECT_TARGET
local COMBATLOG_FRIENDLY = COMBATLOG_OBJECT_REACTION_FRIENDLY
local COMBATLOG_HOSTILE = COMBATLOG_OBJECT_REACTION_HOSTILE
local COMBATLOG_PLAYER = COMBATLOG_OBJECT_TYPE_PLAYER

local SpellCastEvents = {
	SPELL_CAST_START = 1,
	SPELL_CAST_SUCCESS = 1,
	SPELL_CREATE = 1,
}

local band = bit.band
function SpellAlerter:COMBAT_LOG_EVENT_UNFILTERED(_, _, eventtype, _, srcName, srcFlags, _, dstName, dstFlags, spellID, spellName, _, auraType)
	if not spellID then return end
	PlayerGUID = PlayerGUID or UnitGUID("player")
	if SpellCastEvents[eventtype] and band(srcFlags, COMBATLOG_HOSTILE) == COMBATLOG_HOSTILE and SpellCasts[spellName] then
		if profile.TargetOnly and band(srcFlags,COMBATLOG_TARGET) ~= COMBATLOG_TARGET then return end
		if profile.SpellCastsPlayersOnly and band(COMBATLOG_PLAYER,srcFlags) ~= COMBATLOG_PLAYER then return end
		self:FormatInfo(match(srcName,"[^-]*"),spellName,select(3,GetSpellInfo(spellID)),SpellCasts[spellName].Sound,SPELLCASTS,unpack(colors[SpellCasts[spellName].Color]))
	elseif eventtype == "SPELL_AURA_APPLIED" then
		if profile.TargetOnly and band(dstFlags,COMBATLOG_TARGET) ~= COMBATLOG_TARGET then return end
		if auraType == "BUFF" and band(dstFlags,COMBATLOG_HOSTILE) == COMBATLOG_HOSTILE and EnemyBuffs[spellName] then
			self:FormatInfo(match(dstName,"[^-]*"),spellName,select(3,GetSpellInfo(spellID)),EnemyBuffs[spellName].Sound,ENEMYBUFFS,unpack(colors[EnemyBuffs[spellName].Color]))
		elseif auraType == "DEBUFF" and band(dstFlags,COMBATLOG_FRIENDLY) == COMBATLOG_FRIENDLY and FriendlyDebuffs[spellName] then
			self:FormatInfo(match(dstName,"[^-]*"),spellName,select(3,GetSpellInfo(spellID)),FriendlyDebuffs[spellName].Sound,FRIENDLYDEBUFFS,unpack(colors[FriendlyDebuffs[spellName].Color]))
		end
	end
end

local function FormatAbility(s)
	s = s:gsub("(%a)(%a*)('*)(%a*)", function (a,b,c,d) return a:upper()..b:lower()..c..d:lower() end)
	s = s:gsub("(The)", string.lower)
	s = s:gsub("(Of)", string.lower)
	return s
end


local function ValidateSpellInfo(info,value)
	local spellid = tonumber(value)
	local exists = false
	if type(spellid) == "number" and GetSpellInfo(spellid) then
		exists = true
	else
		value = FormatAbility(value)
		for id=65000,1,-1 do
			local name = GetSpellInfo(id)
			if name and name == value then 
				exists = true; break
			end
		end
	end
	return exists and true or L["Invalid spell name/spell id"]
end

local function IsDisabled()
	return not SpellAlerter.db.global.Enabled
end

function SpellAlerter:InitializeOptions()
	self.options = {
		type = "group",
		name = L["Spell Alerter"],
		order = 100,
		args = {
			About = {
				type = "group",
				name = L["About"],
				order = -1,
				args = {
					author = {
						type = "description",
						name = [=[Author: |cffffd200Kollektiv|r

Find updates on |cffffff78http://www.arenacast.net|r]=],
						order = 100,
					},
				},
			},
			Enabled = {
				type = "toggle",
				name = L["Enabled"],
				order = 25,
				get = function() return self.db.global.Enabled end,
				set = function(info,value) self.db.global.Enabled = value; self:SetEnabledState(value); if value then self:Enable() else self:Disable() end end,
				width = "half",
			},
			Lock = {
				type = "toggle",
				name = L["Lock"],
				set = function(info,value) self.db.global.Lock = value; self:ApplyLock() end,
				get = function(info) return self.db.global.Lock end,
				order = 55,
				disabled = IsDisabled,
				width = "half",
			},
			ShowMinimap = {
				type = "toggle",
				name = L["Minimap"],
				set = function(info,value) self.db.global.Minimap.hide = not value; self:ApplyMinimap() end,
				get = function(info) return not self.db.global.Minimap.hide end,
				order = 56,
				disabled = IsDisabled,
				width = "half",
			},
			Test = {
				type = "execute",
				name = L["Test"],
				order = 60,
				func = function() self:FormatInfo(UnitName("player"),"Earth Shield","Interface\\Icons\\Spell_Nature_SkinofEarth","Bell Toll Alliance",SPELLCASTS,0,1,0,"Kollektor") end,
				width = "half",
				disabled = IsDisabled,
			},
			Version = {
				type = "description",
				name = "|cff00ff00"..L["Version"].."|r: "..GetAddOnMetadata("SpellAlerter","Version"),
				order = 80,
			},
			
			Settings_Group = {
				type = "group",
				name = L["Settings"],
				order = 100,
				get = function(info) return profile[info[#info]] end,
				set = function(info,value) profile[info[#info]] = value; self:ApplySettings() end,
				disabled = IsDisabled,
				args = {
					Toggles_Group = {
						type = "group",
						name = L["Toggles"],
						order = 100,
						inline = true,
						args = {
							General_Group = {
								type = "group",
								name = L["General"],
								order = 100,
								args = {
									ShowIcon = {
										type = "toggle",
										name = L["Show icon"],
										order = 200,
									},
									SpellNames = {
										type = "toggle",
										name = L["Show spell names"],
										order = 300,
									},
									TargetOnly = {
										type = "toggle",
										name = L["Target only"],
										order = 350,
									},
									ClassColored = {
										type = "toggle",
										name = L["Class colored"],
										desc = L["Is not 100% reliable outside of arena. Overrides colors in Spell List"],
										order = 400,
									},
								},
							},
							Filters_Group = {
								type = "group",
								name = L["Filters"],
								order = 150,
								inline = true,
								args = {
									SpellCasts_Filter_Group = {
										type = "group",
										name = L["Spell Casts"],
										order = 100,
										inline = true,
										args = {
											SpellCastsPlayersOnly = {
												type = "toggle",
												name = L["Players only"],
												order = 100,
											},
										},
									},
								},
							},
						},
					},
					
					Sliders_Group = {
						type = "group",
						name = L["Sliders"],
						order = 200,
						inline = true,
						args = {
							HoldTime = {
								type = "range",
								name = L["Hold time"],
								order = 400,
								min = 0.5,
								max = 5,
								step = 0.1,
							},
							IconSize = {
								type = "range",
								name = L["Icon size"],
								order = 500,
								min = 10,
								max = 100,
								step = 1,
							},
							FontSize = {
								type = "range",
								name = L["Font size"],
								order = 600,
								min = 10,
								max = 40,
								step = 1,
							},
						},
					},
					LookAndFeel_Group = {
						type = "group",
						name = L["Look and feel"],
						order = 300,
						inline = true,
						args = {
							Font = {
								type = "select",
								name = L["Font"],
								order = 100,
								values = function()
									self.fonts = self.fonts or {}
									wipe(self.fonts)
									for _, name in pairs(SM:List("font")) do self.fonts[name] = name end
									return self.fonts
								end,
							},
							YouText = {
								type = "input",
								name = L["Replace player name with..."],
								order = 200,
							},
						},
					},
					ShowTarget = {
						type = "toggle",
						name = L["Show target"],
						order = 350,
					},
					Target_Group = {
						type = "group",
						name = L["Targets"],
						order = 400,
						disabled = function() return not profile.ShowTarget or IsDisabled() end,
						inline = true,
						args = {
							TargetGraphic = {
								type = "select",
								name = L["Target graphic"],
								order = 900,
								values = {
									arrow = L["Arrow"],
									text = L["Text"],
								},
							},
						},
					},
				},
			},
			Spell_List_Group = {
				type = "group",
				name = L["Spell List"],
				order = 200,
				disabled = IsDisabled,
				args = {
					CategorySelect = {
						type = "select",
						name = L["Category"],
						order = 50,
						set = function(info,value) ListSelect = nil; profile.CategorySelect = value end,
						get = function() return profile.CategorySelect end,
						values = CategoryValues,
					},
					Category_Header = {
						type = "header",
						name = "",
						order = 60,
					},
					Add_Desc = {
						type = "description",
						name = L["Enter a spell name/spell id. Spell names are automatically case formatted."],
						order = 70,
					},
					Add_Editbox = {
						type = "input",
						name = L["Add"],
						get = function() return "" end,
						set = function(info,value) 
							if tonumber(value) then value = GetSpellInfo(value) 
							else value = FormatAbility(value) end
							profile[profile.CategorySelect][value] = {Color = "RED"}
							ListSelect = value
						end,
						validate = ValidateSpellInfo,
						order = 120,
					},
					Blank1 = {
						type = "description",
						name = "",
						order = 130,
					},
					List_Select_Group = {
						type = "group",
						name = function() return CategoryValues[profile.CategorySelect].." "..L["List"] end,
						order = 140,
						inline = true,
						args = {
							List_Select = {
								type = "select",
								name = "",
								order = 140,
								get = function() return ListSelect end,
								set = function(info,value) ListSelect = value end,
								values = function() 
									self.listTemp = self.listTemp or {}
									wipe(self.listTemp)
									for k,v in pairs(profile[profile.CategorySelect]) do
										self.listTemp[k] = colorHexes[v.Color]:sub(1,10)..k.."|r"
									end
									return self.listTemp
								end,
								dialogControl = "SA_FauxScrollFrame",
							},
							Delete = {
								type = "execute",
								name = L["Delete"],
								order = 145,
								func = function()
									profile[profile.CategorySelect][ListSelect] = nil
									ListSelect = nil
								end,
								disabled = function() return not ListSelect or IsDisabled() end,
								width = "half",
							},
							Attributes_Group = {
								type = "group",
								name = function()
									if not ListSelect then
										return L["Attributes"] 
									else
										return L["Attributes"] .." of "..ListSelect
									end
								end,
								order = 250,
								inline = true,
								set = function(info,value)
									if not ListSelect then return end
									profile[profile.CategorySelect][ListSelect][info[#info]] = value
								end,
								get = function(info)
									if not ListSelect then return end
									return profile[profile.CategorySelect][ListSelect][info[#info]]
								end,
								disabled = function() return not ListSelect or IsDisabled() end,
								args = {
									Color = {
										type = "select",
										name = L["Color"],
										order = 250,
										values = colorHexes,
									},
									Sound = {
										type = "select",
										name = L["Sound"],
										order = 300,
										values = function()
											self.sounds = self.sounds or {}
											wipe(self.sounds)
											for _, name in pairs(SM:List("sound")) do self.sounds[name] = name end
											return self.sounds	
										end,
										dialogControl = "LSM30_Sound",
									},
								},
							},
						},
					},
				},
			},
			Target_Ignore_Group = {
				type = "group",
				name = L["Ignores"],
				order = 300,
				disabled = IsDisabled,
				args = {
					desc1 = {
						type = "description",
						name = L["These are Spell Casts that won't show targets\n"],
						order = 100,
					},
					Add_Desc = {
						type = "description",
						name = L["Enter a spell name/spell id. Spell names are automatically case formatted."],
						order = 150,
					},
					Add_Editbox = {
						type = "input",
						name = L["Add"],
						get = function() return "" end,
						set = function(info,value) 
							if tonumber(value) then value = GetSpellInfo(value) 
							else value = FormatAbility(value) end
							Ignores[value] = value
							ListSelect2 = value
						end,
						validate = ValidateSpellInfo,
						order = 175,
					},
					blank = { type = "description", name = "", order = 190},
					List_Select2 = {
						type = "select",
						name = "",
						order = 200,
						get = function() return ListSelect2 end,
						set = function(info,value) ListSelect2 = value end,
						values = function() return Ignores end,
						dialogControl = "SA_FauxScrollFrame",
					},
					Delete = {
						type = "execute",
						name = L["Delete"],
						order = 250,
						func = function()
							profile.Ignores[ListSelect2] = nil
							ListSelect2 = nil
						end,
						disabled = function() return not ListSelect2 or IsDisabled() end,
						width = "half",
					},
				},
			},
		},
	}
end


_G.SpellAlerter = SpellAlerter
